/* header file for task manager */
/* ptm.h */


/* Status of Task */

#define PENDING -1
#define READY 0
#define RUNNING 1
#define COMPLETED 2


/* Data Type */

#define INT 0
#define REAL 1
#define CHAR 2
#define NAME 3


/* limits of arrays */

#define MAXTASK 16
#define MAXVAR 32
#define MAXPARA 8
#define MAXSTR 64


/* message type */

#define hMSGSEND 0
#define hMSGRECV 1
#define hMSGABORT 999


/* system variables */

int hTID; /* tid from which the message is originated */
int hARGC; /* the number of arguments */
int hRETC; /* the number of results */
int hNPROC; /* the number of tasks to be spawned */
int hTIDS[MAXTASK]; /* the tids of tasks spawned */
double hTIME; /* elapse time in second */
double hSTARTTIME; /* starting time of the whole appliction */
double hENDTIME; /* ending time of the whole application */

/* Definition of Data Structures */

struct htask {
   char name[32]; /* name of task */
   char host[32]; /* name of host */
   int nproc; /* number of copies of task to be spawned */
   int arch; /* spawn task to architecture or particular host */
   int icntr; /* number of input parameters */
   int ocntr; /* number of output parameters */
   int fcntr; /* number of filenames to be received by task */
   int input[MAXPARA]; /* pointer to variables as input parameters */
   int output[MAXPARA]; /* pointer to variables as output parameters */
   int status; /* -1=pending 0=ready, 1=running, 2=completed  */
   int tid; /* tid of the task */
   double start; /* starting time of task */
   double end; /* ending time of task */
   } hTASK[MAXTASK];

struct hvariable { /* all variables are in form of an one-dimension array */
   char name[9];   /* but with size in two dimensions */
   int type; /* 0=int, 1=real, 2=char */
   int rows; /* number of rows ofrray */
   int cols; /* number of cols of array */
   int *iptr; /* pointer to inter array */
   float *rptr; /* pointer to real array */
   char cptr[MAXSTR]; /* pointer to string */
   } hVAR[MAXVAR];

struct hphase { /* the phase of execution */
   int task[MAXTASK]; /* tasks to be executed in each phase */
   int result[MAXVAR]; /* variables available after each phase */
   int dealloc[MAXVAR]; /* variables to be freed after each phase */
   int tcntr; /* number of tasks to be executed in each phase */
   int rcntr; /* number of variables available after each phase */
   int dcntr; /* number of variables to be freed after each phase */
   } hPHASE[MAXTASK];

struct hargv {
   int type; /* type of argument */
   int dims; /* number of dimension */
   int rows; /* number of rows */
   int cols; /* number of cols */
   int *iptr; /* integer array pointer - 1 dimensional */
   float *rptr; /* real array pointer - 1 dimensional */
   char cptr[MAXSTR]; /* char array pointer */
} hARGV[MAXPARA], hRETV[MAXPARA];


/* Definition of Macros */

#define hSEND() \
{ \
   int hI,hSIZE; \
   pvm_initsend(PvmDataDefault); \
   pvm_pkint(&hNPROC,1,1); \
   pvm_pkint(hTIDS,hNPROC,1); \
   pvm_pkint(&hARGC,1,1); \
   for (hI=0;hI<hARGC;hI++) { \
      hSIZE = hARGV[hI].rows * hARGV[hI].cols; \
      pvm_pkint(&hARGV[hI].type,1,1); \
      pvm_pkint(&hARGV[hI].rows,1,1); \
      pvm_pkint(&hARGV[hI].cols,1,1); \
      switch(hARGV[hI].type) \
      { \
      case INT: \
       pvm_pkint(hARGV[hI].iptr,hSIZE,1); \
       break; \
      case REAL: \
       pvm_pkfloat(hARGV[hI].rptr,hSIZE,1); \
       break; \
      case CHAR: \
       pvm_pkstr(hARGV[hI].cptr); \
       break; \
      case NAME: \
       pvm_pkstr(hARGV[hI].cptr); \
       break; \
      } \
   } \
   pvm_mcast(hTIDS,hNPROC,hMSGSEND); \
}


#define hRECV() \
{ \
   int hSIZE,hI,hMSG; \
   hMSG = pvm_recv(-1,hMSGRECV); \
   pvm_bufinfo(hMSG,(int*)0,(int*)0,&hTID); \
   pvm_upkint(&hRETC,1,1); \
   if (hRETC<0) { \
      printf("[PTM]: Task [%x] aborted\n",hTID); \
      exit(1); \
   } \
   for (hI=0;hI<hRETC;hI++) \
   { \
      pvm_upkint(&hRETV[hI].type,1,1); \
      pvm_upkint(&hRETV[hI].rows,1,1); \
      pvm_upkint(&hRETV[hI].cols,1,1); \
      hSIZE = hRETV[hI].rows*hRETV[hI].cols; \
      switch(hRETV[hI].type) \
      { \
      case INT: \
       hRETV[hI].iptr = (int *)calloc(hSIZE,sizeof(int)); \
       pvm_upkint(hRETV[hI].iptr,hSIZE,1); \
       break; \
      case REAL: \
       hRETV[hI].rptr = (float *)calloc(hSIZE,sizeof(float)); \
       pvm_upkfloat(hRETV[hI].rptr,hSIZE,1); \
       break; \
      case CHAR: \
       pvm_upkstr(hRETV[hI].cptr); \
       break; \
      } \
   } \
}



